import { defineStore } from "pinia";

import Institutes from "./data/dfnaai.json";

import type { DFNAAIData, DFNAAIInstitution, LoginState } from "./types";

// import the main store
import useMainStore from "@/store/index";
import useNotificationStore from "@/store/notification";
import { useLocalStorage } from "@vueuse/core";
import { AccountApi, SelfApi, TosApi } from "@coscine/api-client";
import type { AxiosError } from "axios";
import axios from "axios";
import { UserTermsOfServiceAcceptDto } from "@coscine/api-client/dist/types/Coscine.Api";

export function getCookieValue(name: string): string | null {
  const cookies = document.cookie.split(";");
  for (const cookie of cookies) {
    const [cookieName, cookieValue] = cookie.trim().split("=");
    if (cookieName === name) {
      return decodeURIComponent(cookieValue);
    }
  }
  return null;
}

/*  
  Store variable name is "this.<id>Store"
    id: "login" --> this.loginStore
  Important! The id must be unique for every store.
*/
export const useLoginStore = defineStore({
  id: "login",
  /*  
    --------------------------------------------------------------------------------------
    STATES
    --------------------------------------------------------------------------------------
  */
  state: (): LoginState => ({
    expiredSession: null,
    currentTosVersion: null,
    loginStoredData: useLocalStorage("coscine.login.storedData", ""), // TODO: Use Type DFNAAIInstitution from March 2023
    loginUrls: null,
  }),

  /*  
    --------------------------------------------------------------------------------------
    GETTERS
    --------------------------------------------------------------------------------------
    Synchronous code only.
    
    In a component use as e.g.:
      :label = "this.loginStore.<getter_name>;
  */
  getters: {
    isLoggedIn(): boolean {
      const mainStore = useMainStore();
      if (
        (mainStore.coscine.authorization.bearer &&
          this.expiredSession !== null &&
          !this.expiredSession) ||
        getCookieValue("CoscineLoginExpiration") !== null
      ) {
        return true;
      } else {
        return false;
      }
    },
    institutions(): DFNAAIData[] {
      return Institutes as DFNAAIData[];
    },
    selectedInstitution(): DFNAAIInstitution | null {
      const storedData = this.loginStoredData;
      if (storedData && storedData.trim() !== "") {
        return JSON.parse(this.loginStoredData) as DFNAAIInstitution;
      } else return null;
    },
    // Id for the orcid external provider
    orcidId(): string {
      return "f383f83c-26cb-47eb-80c8-639ed05bd0de";
    },
    // Id for the NFDI4Ing AAI external provider
    nfdi4IngAaiId(): string {
      return "3a1e7185-0328-47ed-afd7-a4ffd7f03a79";
    },
    // Calculate, how long the cookie session is valid in miliseconds
    getCookieLoginExpiration(): number | null {
      const cookieValue = getCookieValue("CoscineLoginExpiration");
      if (cookieValue) {
        const expirationDate = new Date(cookieValue);
        const now = new Date();
        const nowUTC = new Date(now.toISOString());
        const remainingTime = expirationDate.getTime() - nowUTC.getTime();
        return remainingTime > 0 ? remainingTime : 0;
      }
      return null;
    },
    isCookieLogin(): boolean {
      return getCookieValue("CoscineLoginExpiration") !== null;
    },
  },
  /*  
    --------------------------------------------------------------------------------------
    ACTIONS
    --------------------------------------------------------------------------------------
    Asynchronous & Synchronous code comes here (e.g. API calls and VueX mutations).
    To change a state use an action.
    
    In a component use as e.g.:
      @click = "this.loginStore.<action_name>();
  */
  actions: {
    logout() {
      // Clear the local storage
      const storedKeys = Object.keys(
        localStorage._store ? localStorage._store : localStorage,
      );
      storedKeys.forEach((key) => {
        if (
          !key.startsWith("coscine.login.storedData") &&
          !key.startsWith("coscine.banner") &&
          !key.startsWith("coscine.theme")
        ) {
          localStorage.removeItem(key);
        }
      });

      // Store, if cookie login was used
      const isCookieLogin = getCookieValue("CoscineLoginExpiration") !== null;

      // Clear the session storage
      sessionStorage.clear();

      // The logout has to be done by the API.
      // For this we have to do a form post out of a function.
      // There is no other way to do it.
      // Don't try or dare to refactor this.
      // I'm serious, don't touch this.
      if (isCookieLogin) {
        const form = document.createElement("form");

        form.method = "post";
        form.action = "/coscine/api/v2/self/session/logout";

        // Append the form to the body
        document.body.appendChild(form);

        // Submit the form
        form.submit();
      } else {
        window.location.href = "/coscine/api/Coscine.Api.STS/account/logout";
      }
    },

    logoutFromShibboleth(institution: DFNAAIInstitution) {
      /*
        This approach was adopted from the previous login app. 
        The URL works for most of the providers, but does not work for RWTH.

        Example: 
          - RWTH Test entityId: https://login-test.rz.rwth-aachen.de/shibboleth  (adding `idp/profile/Logout` DOES NOT work)
          - Actual Test URL: https://sso-test.rwth-aachen.de/  (adding `idp/profile/Logout` works)

        TODO: Get the proper logout URL for the entityId
      */
      window.location.href = institution.entityId + "/idp/profile/Logout";
    },

    async getOrcidLoginStatus(logUserOut = false): Promise<boolean> {
      const notificationStore = useNotificationStore();
      try {
        /*
          This does not seem to work. The GET request always delivers a "false", but running it in the browser, "true" is returned when logged in.

          TODO: Figure this out.
        */
        await axios
          .get(
            `https://orcid.org/userStatus.json${
              logUserOut ? "?logUserOut=true" : ""
            }`,
            { withCredentials: true },
          )
          .then((result) => {
            return result.data["loggedIn"] as boolean;
          });
        return false;
      } catch (error) {
        // Handle other Status Codes
        notificationStore.postApiErrorNotification(error as AxiosError);
        return false;
      }
    },

    retrieveSessionStatus() {
      if (this.expiredSession === null) {
        const mainStore = useMainStore();
        const duration = mainStore.getCurrentTokenExpirationDuration();
        if (duration > 0) {
          this.expiredSession = false;
        }
      }
    },

    async retrieveCurrentTosVersion() {
      const notificationStore = useNotificationStore();
      try {
        const apiResponse = await TosApi.getTos();
        this.currentTosVersion = apiResponse.data.data;
      } catch (error) {
        // Handle other Status Codes
        notificationStore.postApiErrorNotification(error as AxiosError);
      }
    },

    async retrieveLoginUrls() {
      const notificationStore = useNotificationStore();
      try {
        const apiResponse = await AccountApi.accountGetLoginUrls();
        this.loginUrls = apiResponse.data;
      } catch (error) {
        // Handle other Status Codes
        notificationStore.postApiErrorNotification(error as AxiosError);
      }
    },

    storeInstitutionSelection(institution: DFNAAIInstitution) {
      this.loginStoredData = JSON.stringify(institution);
    },

    async acceptToS(): Promise<boolean> {
      const notificationStore = useNotificationStore();
      try {
        // Retrieve the current TOS version if missing
        if (!this.currentTosVersion?.version) {
          await this.retrieveCurrentTosVersion();
        }
        // Build the DTO and send it
        const userTermsOfServiceAcceptDto: UserTermsOfServiceAcceptDto = {
          version: this.currentTosVersion?.version ?? "",
        };
        await SelfApi.acceptCurrentTos({
          userTermsOfServiceAcceptDto: userTermsOfServiceAcceptDto,
        });
        return true;
      } catch (error) {
        // Handle other Status Codes
        notificationStore.postApiErrorNotification(error as AxiosError);
        return false;
      }
    },
  },
});

export default useLoginStore;
