import { useLocalStorage } from "@vueuse/core";
import { defineStore } from "pinia";
import type { Router, RouteLocationNormalized } from "vue-router";
import type { Theme, MainState } from "./types";
import { v4 as uuidv4 } from "uuid";
import * as jose from "jose";
import moment from "moment";

import { removeQueryParameterFromUrl } from "@/router";

import useLoginStore from "@/modules/login/store";

/*  
  Store variable name is "this.<id>Store"
    id: "main" --> this.mainStore
  Important! The id must be unique for every store.
*/
export const useMainStore = defineStore({
  id: "main",
  /*  
    --------------------------------------------------------------------------------------
    STATES
    --------------------------------------------------------------------------------------
  */
  state: (): MainState => ({
    coscine: {
      authorization: {
        bearer: useLocalStorage("coscine.authorization.bearer", ""),
      },
      clientcorrelation: {
        id: useLocalStorage("coscine.clientcorrelation.id", uuidv4()),
      },
      loading: {
        counter: 0,
      },
      locale: useLocalStorage("coscine.locale", "en"),
      theme: useLocalStorage<Theme>("coscine.theme", "light"),
    },
    sidebarActive: useLocalStorage("coscine.sidebar.active", true),
  }),

  /*  
    --------------------------------------------------------------------------------------
    GETTERS
    --------------------------------------------------------------------------------------
    Synchronous code only.
    
    In a component use as e.g.:
      :label = "this.mainStore.<getter_name>;
  */
  getters: {
    currentTheme(): Theme {
      return this.coscine.theme;
    },
    isLoading(): boolean {
      return this.coscine.loading.counter > 0;
    },
    isLoggedIn(): boolean {
      const loginStore = useLoginStore();
      if (this.coscine.authorization.bearer && !loginStore.expiredSession) {
        return true;
      } else {
        return false;
      }
    },
  },
  /*  
    --------------------------------------------------------------------------------------
    ACTIONS
    --------------------------------------------------------------------------------------
    Asynchronous & Synchronous code comes here (e.g. API calls and VueX mutations).
    To change a state use an action.

    In a component use as e.g.:
      @click = "this.mainStore.<action_name>();
  */
  actions: {
    setAccessTokenFromRoute(router: Router, route: RouteLocationNormalized) {
      // .../?accesstoken=<token>
      const acccessTokenName = "accesstoken";
      if (route.query[acccessTokenName]) {
        this.coscine.authorization.bearer =
          route.query[acccessTokenName].toString();

        if (router.currentRoute.value.name === "login") {
          router.push({ name: "home" });
        }
        // Remove accesstoken from the URL
        removeQueryParameterFromUrl(route, acccessTokenName);
      }
    },

    setTheme(theme: Theme) {
      this.coscine.theme = theme;
    },

    decreasePageLoadingContent() {
      this.coscine.loading.counter--;
    },

    increasePageLoadingContent() {
      this.coscine.loading.counter++;
    },

    getCurrentTokenExpirationDuration(): number {
      const loginStore = useLoginStore();

      const remainingCookieDuration = loginStore.getCookieLoginExpiration;
      if (remainingCookieDuration !== null) {
        return remainingCookieDuration;
      }

      try {
        const jwt = jose.decodeJwt(this.coscine.authorization.bearer);
        const now = moment.utc(moment.now());
        if (jwt.exp) {
          // Use UTC to avoid time conversion errors
          const tokenExpiresAt = moment(jwt.exp * 1000).utc();
          const untilTokenExpiration = moment.duration(
            tokenExpiresAt.diff(now),
          );

          // Return as milliseconds for setInterval() method
          return untilTokenExpiration.asMilliseconds();
        } else {
          return 0;
        }
      } catch (error) {
        // Provided token is not a valid JWT Token
        return 0;
      }
    },
  },
});

export default useMainStore;
